% For use with Matlab
% Copyright (C) 2019 -- Jerome Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araujo <rui@isr.uc.pt>
% Jerome Mendes, Francisco Souza, Rui Araujo, and Saeid Rastegar. "Neo-fuzzy neuron learning using backfitting algorithm". Neural Computing and Application, 2019.
% The "GAM-ZOTS toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jerome Mendes, Francisco Souza, Rui Araujo, and Saeid Rastegar. 
% Neo-fuzzy neuron learning using backfitting algorithm.
% Neural Computing and Applications, 31(8):3609-3618, August 2019.
% DOI: https://doi.org/10.1007/s00521-017-3301-4
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

function [BestNumber_Rules] = C_BestNumberRules(n_rules_to_train, data_ytrain, data_xtrain, limit_it, limit_theta, dataTotal_x, per_extra)
    %% Description
    % This function returns the best number of rules from the candidate ones (n_rules_to_train)
    %% Function inputs
    % n_rules_to_train - Number of rules which will be used on the training
    % data_ytrain - Train dataset of output variable
    % data_xtrain - Train dataset of input variables
    % limit_it - maximal number of learning interactions - variable lim_{it}
    % limit_theta - Termination condition - variable \epsilon
    % dataTotal_x - All dataset of input variables
    % per_extra - percentage extra to increase the limits of the universe of
    %% Function output
    % BestNumber_Rules - Best number of rules found

%% Cross-validation partition for train dataset
n_partition = 10; % number of partitions

cv = cvpartition(length(data_ytrain),'kfold',n_partition);

%% Train for all number of rules defined by n_rules_to_train
for n_rules_aux = n_rules_to_train % for all defined number of rules
    
    fprintf('Running n_rules= %d\n', n_rules_aux); % just to print

    for k = 1:n_partition % Cross-validation
        
        xx_train = data_xtrain(cv.training(k),:); % train input dataset
        xx_val = data_xtrain(cv.test(k),:); % validation input dataset
        
        yy_train = data_ytrain(cv.training(k),:); % train output dataset 
        yy_val = data_ytrain(cv.test(k),:); % validation output dataset
        
        x = C_FAM ( yy_train, xx_train, limit_it, limit_theta,dataTotal_x, per_extra, n_rules_aux); % learning algorithm

        yy_val_est = FinalyModel_C(x, xx_val); % estimation of the learned neo-fuzzy

        mse_cross_val(n_rules_aux-1,k) = nrmse(yy_val, yy_val_est'); % normalized root-mean-squared error - NRMSE

    end
    
    mean_mse_cross_val(n_rules_aux-1) = mean( mse_cross_val(n_rules_aux-1,:) ); % mean of the NRMSE
       
end

Find_Number_Rules = find( mean_mse_cross_val(:)== min(  mean_mse_cross_val(:) ) ) + 1;

BestNumber_Rules =   Find_Number_Rules(1); % For the case of multiple Rule selected

end