% For use with Matlab
% Copyright (C) 2019 -- Jerome Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araujo <rui@isr.uc.pt>
% Jerome Mendes, Francisco Souza, Rui Araujo, and Saeid Rastegar. "Neo-fuzzy neuron learning using backfitting algorithm". Neural Computing and Application, 2019.
% The "GAM-ZOTS toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jerome Mendes, Francisco Souza, Rui Araujo, and Saeid Rastegar. 
% Neo-fuzzy neuron learning using backfitting algorithm.
% Neural Computing and Applications, 31(8):3609-3618, August 2019.
% DOI: https://doi.org/10.1007/s00521-017-3301-4
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

function result = init_script()
    %% Description
    % This function presents the inial GUI, in which, the main parameters
    % are configured, and the train and test datasets are defined.
    
S.in{1,1} = '100';
S.in{2,1} = '0.001';
S.in{3,1} = '2';
S.in{4,1} = 'housing_train.mat';
S.in{5,1} = 'housing_test.mat';
S.hint{1,1} = 'Maximum number of learning interactions.'; % lim_it
S.hint{2,1} = 'Termination condition defined on Algorithm 2 Step 3.c.ii.'; % epsilon
S.hint{3,1} = 'Number of fuzzy rules per input variable.'; % n_rules
S.hint{4,1} = 'Dataset for training: [x1 x2 ... xn y]'; % Train
S.hint{5,1} = 'Dataset for testing: [x1 x2 ... xn y]'; % Test

S.fh = figure('units','pixels',...
              'position',[560 200 400 493],...
              'menubar','none',...
              'name','GAM-ZOTS Toolbox',...
              'numbertitle','off',...
              'visible','off',...
              'resize','off');

S.hpCfg = uipanel(S.fh,'Title','Parameters Config:','FontSize',11,...
              'units','normalized',...
              'BackgroundColor',[.94 .94 .94],...
              'Position',[0.1 0.6 0.8 0.35],...
              'visible','on');

S.hpLoad = uipanel(S.fh,'Title','Load Dataset:','FontSize',11,...
              'units','normalized',...
              'Position',[0.1 0.15 0.8 0.3],...
              'visible','on');

S.htxt1 = uicontrol(S.hpCfg,'Style','text', 'String','  lim_it',...
              'units','normalized',...
              'Fontsize',12,...
              'TooltipString', S.hint{1,1},...
              'Position',[0.16 0.72 0.2 0.13]);

S.htxt2 = uicontrol(S.hpCfg,'Style','text', 'String','epsilon',...
              'units','normalized',...
              'Fontsize',12,...
              'TooltipString', S.hint{2,1},...
              'Position',[0.16 0.41 0.2 0.13]);

S.htxt3 = uicontrol(S.hpCfg,'Style','text', 'String','n_rules',...
              'units','normalized',...
              'Fontsize',12,...
              'TooltipString', S.hint{3,1},...
              'Position',[0.16 0.1 0.2 0.13]);

S.htxt4 = uicontrol(S.hpLoad,'Style','text', 'String','Train',...
              'units','normalized',...
              'Fontsize',12,...
              'TooltipString', S.hint{4,1},...
              'Position',[0.06 0.64 0.13 0.13]);

S.htxt5 = uicontrol(S.hpLoad,'Style','text', 'String',' Test',...
              'units','normalized',...
              'Fontsize',12,...
              'TooltipString', S.hint{5,1},...
              'Position',[0.06 0.27 0.13 0.13]);

S.edtLimit = uicontrol(S.hpCfg,'Style','edit','String',S.in{1,1},...
              'units','normalized',...
              'Fontsize',12,...
              'TooltipString', S.hint{1,1},...
              'Position',[0.4 0.715 0.35 0.15]);

S.edtEpsilon = uicontrol(S.hpCfg,'Style','edit','String',S.in{2,1},...
              'units','normalized',...
              'Fontsize',12,...
              'TooltipString', S.hint{2,1},...
              'Position',[0.4 0.405 0.35 0.15]);

S.edtNRules1 = uicontrol(S.hpCfg,'Style','edit','String',S.in{3,1},...
              'units','normalized',...
              'Fontsize',12,...
              'TooltipString', S.hint{3,1},...
              'Position',[0.4 0.095 0.35 0.15]);

S.edtLoadTrain = uicontrol(S.hpLoad,'Style','edit','String',S.in{4,1},...
              'units','normalized',...
              'Fontsize',12,...
              'TooltipString', S.hint{4,1},...
              'Position',[0.2 0.61 0.48 0.18]);

S.edtLoadTest = uicontrol(S.hpLoad,'Style','edit','String',S.in{5,1},...
              'units','normalized',...
              'Fontsize',12,...
              'TooltipString', S.hint{5,1},...
              'Position',[0.2 0.23 0.48 0.18]);

S.cancel = uicontrol(S.fh,'Style','edit','String','true',...
              'visible','off');

S.hbtn = uicontrol('Style','push', 'String','Run',...
              'Fontsize',12,...
              'Position',[152 29 100 25.6],...
              'callback',{@btnRun_Callback,S});

S.btnLoadTrain = uicontrol(S.hpLoad,'Style','push', 'String','load...',...
              'units','normalized',...
              'Fontsize',12,...
              'Position',[0.695 0.61 0.22 0.18],...
              'TooltipString', 'Click to load train dataset',...
              'callback',{@btnLoadTrain_Callback,S});

S.btnLoadTest = uicontrol(S.hpLoad,'Style','push', 'String','load...',...
              'units','normalized',...
              'Fontsize',12,...
              'Position',[0.695 0.23 0.22 0.18],...
              'TooltipString', 'Click to load test dataset',...
              'callback',{@btnLoadTest_Callback,S});

set(S.fh,'Visible','on');
uiwait(S.fh);
if ishandle(S.fh)
    if strcmp(get(S.cancel,'string'), 'false')
        result.Limit = str2double(get(S.edtLimit,'string'));
        result.epsilon = str2double(get(S.edtEpsilon,'string'));
        result.n_rules_ini = str2double(get(S.edtNRules1,'string'));
        result.Train = importdata(get(S.edtLoadTrain,'string'));
        result.Test = importdata(get(S.edtLoadTest,'string'));
        result.cancelled = false;
    else
        result.cancelled = true;
    end
    delete(S.fh)
else
    result.Limit = '';
    result.epsilon = '';
    result.n_rules_ini = '';
    result.Extra = '';
    result.Train = '';
    result.Test = '';
    result.cancelled = true;
end
return;


% --- Executes on button press in btnRun.
function [] = btnRun_Callback(varargin)
if checkData(varargin{1,3})
    set(varargin{1, 3}.cancel,'string','false');
    uiresume(varargin{1, 3}.fh);
end


% --- Executes on button press in btnLoadTrain.
function btnLoadTrain_Callback(varargin)
[file,path] = uigetfile('*.mat');
if isequal(file,0)
    set(varargin{1, 3}.edtLoadTrain,'string','');
else % varargin{1, 3}.edtLoadTrain.String  
    set(varargin{1, 3}.edtLoadTrain,'string',fullfile(path,file));
end


% --- Executes on button press in btnLoadTest.
function btnLoadTest_Callback(varargin)
[file,path] = uigetfile('*.mat');
if isequal(file,0)
    set(varargin{1, 3}.edtLoadTest,'string','');
else
    set(varargin{1, 3}.edtLoadTest,'string',fullfile(path,file));
end

% check data consistency
function result = checkData(S)
result = false;
CreateStruct.Interpreter = 'tex';
CreateStruct.WindowStyle = 'modal';
Limit = str2double(get(S.edtLimit,'string'));
epsilon = str2double(get(S.edtEpsilon,'string'));
n_rules_ini = str2double(get(S.edtNRules1,'string'));
% check fields empty
if isnan(Limit) || isnan(epsilon) || isnan(n_rules_ini)||...
   isempty(S.edtLoadTrain.String) || isempty(S.edtLoadTest.String)
    msgbox('All fieds should be filled','Error','error',CreateStruct);
    return;
end
% check Limit consistency
if Limit < 1 || mod(Limit,1)~=0
    msgbox('{\bf lim\_it} should be an integer greater than 0','Error','error',CreateStruct);
    return;
end
% check epsilon consistency
if epsilon < 0
    msgbox('{\bf epsilon} should be greater than or equal 0','Error','error',CreateStruct);
    return;
end
% check n_rules_ini consistency
if n_rules_ini < 2 || mod(n_rules_ini,1)~=0
    msgbox('{\bf n\_rules\_ini} should be an integer greater than or equal 2','Error','error',CreateStruct);
    return;
end
if ~exist(get(S.edtLoadTrain,'string'),'file')
    msgbox('{\bf Train file} should exists','Error','error',CreateStruct);
    return;
end
if ~exist(get(S.edtLoadTest,'string'),'file')
    msgbox('{\bf Test file} should exists','Error','error',CreateStruct);
    return;
end
result = true;
return;