% For use with Matlab
% Copyright (C) 2019 -- Jerome Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araujo <rui@isr.uc.pt>
% Jerome Mendes, Francisco Souza, Rui Araujo, and Saeid Rastegar. "Neo-fuzzy neuron learning using backfitting algorithm". Neural Computing and Application, 2019.
% The "GAM-ZOTS toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jerome Mendes, Francisco Souza, Rui Araujo, and Saeid Rastegar. 
% Neo-fuzzy neuron learning using backfitting algorithm.
% Neural Computing and Applications, 31(8):3609-3618, August 2019.
% DOI: https://doi.org/10.1007/s00521-017-3301-4
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

%% Clean everything
clc
clear
close all
warning('off')

%% add path
addpath(genpath('./GUI'));
addpath(genpath('./Functions'));

%% GUI - Graphical User Interface
gui = init(); 
if (gui.cancelled == 1); return; end

%% configuration file
ParametersConfig 

%% Configuration of the Dataset
% DatasetConfig_manually % Manually dataset definition
DatasetConfig_gui % Dataset definition by the GUI
        
%% Choose the best number of rules from the candidate ones (n_rules_to_train)
n_rules_best = C_BestNumberRules(n_rules_to_train, data_ytrain, data_xtrain,  limit_it, limit_theta, dataTotal_x,per_extra);

%% Learning algorithm
x = C_FAM ( data_ytrain, data_xtrain, limit_it, limit_theta, dataTotal_x, per_extra, n_rules_best);

%% Estimation - Train
y_train_est = FinalyModel_C(x, data_xtrain);
error_train = nrmse(data_ytrain, y_train_est');% normalized root-mean-squared error - NRMSE

%% Estimation - Test
y_test_est = FinalyModel_C(x, data_xtest);
error_test = nrmse(data_ytest, y_test_est');% normalized root-mean-squared error - NRMSE


%% Plots
% Display results on terminal
fprintf('\nFinal Results:\n -Best number of rules: %d\n -Training error: %d\n -Testing error: %d\n', n_rules_best, error_train, error_test); % just to print
% final GUI, final results are presented, and also estimated output and MF's plots
finish(error_train,error_test,n_rules_best,x,y_train_est,data_ytrain,y_test_est,data_ytest);


